"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.teardown = exports.setup = void 0;
const tslib_1 = require("tslib");
const jsdom_1 = require("jsdom");
const utils_1 = require("../utils");
/**
 * Tactical patch for JSDOM dialog issues
 * {@link https://github.com/jsdom/jsdom/issues/3294}
 */
function mockDialog(jsdomWindow) {
    jsdomWindow.HTMLDialogElement.prototype.show = function () {
        this.open = true;
    };
    jsdomWindow.HTMLDialogElement.prototype.showModal = function () {
        this.open = true;
    };
    jsdomWindow.HTMLDialogElement.prototype.close = function (returnValue) {
        this.open = false;
        this.returnValue = returnValue;
    };
    return jsdomWindow.HTMLDialogElement;
}
/**
 * Jsdom setup.
 * @public
 */
const setup = () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const { window } = new jsdom_1.JSDOM('<!DOCTYPE html><html lang="en"><head></head><body></body></html>', {
        pretendToBeVisual: true,
        url: 'https://localhost',
    });
    window.matchMedia = () => ({
        matches: false,
        addListener: () => { },
        removeListener: () => { },
    });
    /**
     * There may have been a change to how the `CSSOM.parse(rule)` works, and how invalid rules are treated. Here we'll be
     * more torrential of such issues given the limitations of JSDOM.
     */
    const cssInsertRule = window.CSSStyleSheet.prototype.insertRule;
    window.CSSStyleSheet.prototype.insertRule = function (rule, index) {
        try {
            return cssInsertRule.call(this, rule, index);
        }
        catch (e) {
            if (rule.includes('@import')) {
                /**
                 * Longstanding issue it seems, https://github.com/jsdom/jsdom/issues/2124
                 */
                utils_1.logger.debug('Please note JSDOM does not support @import, skipping insertion.');
            }
            else {
                /**
                 * Unknown malformation.
                 */
                utils_1.logger.debug('The CSS rule you are inserting is likely malformed', rule);
            }
            /**
             * Return index as if were successful.
             */
            return index;
        }
    };
    window.ResizeObserver = class ResizeObserver {
        constructor(callback) {
            this.callback = callback;
        }
        observe() { }
        disconnect() { }
        triggerResize(width, height) {
            this.callback([{ contentRect: { width, height } }]);
        }
    };
    globalThis.window = window;
    globalThis.customElements = window.customElements;
    globalThis.CSSImportRule = window.CSSImportRule;
    globalThis.CSSMediaRule = window.CSSMediaRule;
    globalThis.CSSRule = window.CSSRule;
    globalThis.CSSStyleDeclaration = window.CSSStyleDeclaration;
    globalThis.CSSStyleRule = window.CSSStyleRule;
    globalThis.CSSStyleSheet = window.CSSStyleSheet;
    globalThis.CustomEvent = window.CustomEvent;
    globalThis.document = window.document;
    globalThis.Document = window.Document;
    globalThis.DocumentFragment = window.DocumentFragment;
    globalThis.HTMLDialogElement = mockDialog(window);
    globalThis.HTMLElement = window.HTMLElement;
    globalThis.HTMLInputElement = window.HTMLInputElement;
    globalThis.HTMLOptionElement = window.HTMLOptionElement;
    globalThis.HTMLSelectElement = window.HTMLSelectElement;
    globalThis.HTMLStyleElement = window.HTMLStyleElement;
    globalThis.SVGElement = window.SVGElement;
    globalThis.KeyboardEvent = window.KeyboardEvent;
    globalThis.localStorage = window.localStorage;
    globalThis.sessionStorage = window.sessionStorage;
    globalThis.MutationObserver = window.MutationObserver;
    globalThis.navigator = window.navigator;
    globalThis.location = window.location;
    globalThis.history = window.history;
    globalThis.Node = window.Node;
    globalThis.Option = window.Option;
    globalThis.requestAnimationFrame = window.requestAnimationFrame;
    globalThis.ShadowRoot = window.ShadowRoot;
    /**
     * Setup utils / environment variables
     */
    globalThis.SOCKET_EXT = 'gwf';
    globalThis.FORCE_HTTP = false;
    globalThis.GENESIS_SOCKET_URL = 'wss://localhost:6060/gwf/';
    globalThis.API_HOST = globalThis.GENESIS_SOCKET_URL;
    globalThis.HTTP_CONFIG = '';
    globalThis.DEFAULT_ORGANISATION = '';
    globalThis.DEFAULT_PASSWORD = '';
    globalThis.DEFAULT_USER = '';
});
exports.setup = setup;
/**
 * Jsdom teardown / cleanup.
 * @public
 * @remarks
 * This strictly is not required, but may be useful for certain test runs. If used, be mindful that you will need to
 * call setup again before the next test run.
 */
const teardown = () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    delete window.matchMedia;
    delete globalThis.window;
    delete globalThis.customElements;
    delete globalThis.CSSImportRule;
    delete globalThis.CSSMediaRule;
    delete globalThis.CSSRule;
    delete globalThis.CSSStyleDeclaration;
    delete globalThis.CSSStyleRule;
    delete globalThis.CSSStyleSheet;
    delete globalThis.CustomEvent;
    delete globalThis.document;
    delete globalThis.Document;
    delete globalThis.DocumentFragment;
    delete globalThis.HTMLDialogElement;
    delete globalThis.HTMLElement;
    delete globalThis.HTMLInputElement;
    delete globalThis.HTMLOptionElement;
    delete globalThis.HTMLSelectElement;
    delete globalThis.HTMLStyleElement;
    delete globalThis.SVGElement;
    delete globalThis.KeyboardEvent;
    delete globalThis.localStorage;
    delete globalThis.MutationObserver;
    delete globalThis.navigator;
    delete globalThis.location;
    delete globalThis.history;
    delete globalThis.Node;
    delete globalThis.Option;
    delete globalThis.requestAnimationFrame;
    delete globalThis.ShadowRoot;
    /**
     * Teardown utils / environment variables
     */
    delete globalThis.SOCKET_EXT;
    delete globalThis.FORCE_HTTP;
    delete globalThis.GENESIS_SOCKET_URL;
    delete globalThis.API_HOST;
    delete globalThis.HTTP_CONFIG;
    delete globalThis.DEFAULT_ORGANISATION;
    delete globalThis.DEFAULT_PASSWORD;
    delete globalThis.DEFAULT_USER;
    /**
     * Cancel all timers etc.
     */
    window.close();
});
exports.teardown = teardown;
/**
 * Run setup on import by default.
 */
(0, exports.setup)();
